////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2008, Lawrence Livermore National Security,
// LLC. Produced at the Lawrence Livermore National Laboratory. 
// Written by Erik Draeger, draeger1@llnl.gov.
//
// LLNL-CODE-432558 All rights reserved.  
//
// This file is part of bigFFT.
//
// bigFFT is distributed under the terms of the GNU General Public License
// as published by the Free Software Foundation, either version 2 of
// the License, or (at your option) any later version.
// See the file COPYING in the root directory of this distribution
// or <http://www.gnu.org/licenses/>.
////////////////////////////////////////////////////////////////////////////////
#ifndef BIGFFT_H
#define BIGFFT_H

#include <stdbool.h>
#include <mpi.h>
#include <complex.h>
#include "fftPrec.h"

#ifdef USE_FFTW
#ifndef __APPLE__
#include <fftw.h>
#else
#include <dfftw.h>
#endif
#endif

typedef struct bigfft_data_struct 
{
  MPI_Comm rowcomm_,colcomm_, comm_;
  int mype_, npes_,myrow_,mycol_,nprow_,npcol_;
  int np0_,np1_,np2_;
  int np0half_;
  int np0loc_,np1locf_,np1locb_,np2loc_;
  int* np0loc_row_;
  int* np1loc_row_;
  int* np1loc_col_;
  int* np2loc_col_;
  int* iprow_y_;
  int* ipcol_z_;
#ifdef USE_FFTW
  fftw_plan fwplan0,fwplan1,fwplan2,bwplan0,bwplan1,bwplan2;
#endif
#ifdef USE_ESSL
  double* aux1xf;
  double* aux1yf;
  double* aux1zf;
  double* aux1xb;
  double* aux1yb;
  double* aux1zb;
  double* aux2;
  int naux1x,naux1y,naux1z,naux2;
#endif
  FftDataType* pack;
  FftDataType* fftin;
  FftDataType* fftout;
  FftDataType* mpibuf;
} BIGFFT_DATA;

int bigfft_init(BIGFFT_DATA *bigdata, MPI_Comm comm, int np0, int np1, int np2, int nprow, int npcol);
int bigfft_info_init(BIGFFT_DATA *bigdata, int np0, int np1, int np2, int nprow, int npcol);
void bigfft_cleanup(BIGFFT_DATA *bigdata);
int bigfft_forward(BIGFFT_DATA *bigdata, double* rhor, double scale);  // in place transform
int bigfft_backward(BIGFFT_DATA *bigdata, double* rhok, double scale); // in place transform
int bigfft_np1loc_fwd(BIGFFT_DATA *bigdata);   // local size of forward rhor block in y-direction
int bigfft_np2loc_fwd(BIGFFT_DATA *bigdata);   // local size of forward rhor block in z-direction
int bigfft_np0loc_bwd(BIGFFT_DATA *bigdata);   // local size of backward rhok block in x-direction
int bigfft_np1loc_bwd(BIGFFT_DATA *bigdata);   // local size of backward rhok block in y-direction
int bigfft_np0loc_row(BIGFFT_DATA *bigdata, int irow);   // local size of forward rhor block in x-direction at row irow
int bigfft_np1loc_row(BIGFFT_DATA *bigdata, int irow);   // local size of forward rhor block in y-direction at row irow
int bigfft_np1loc_col(BIGFFT_DATA *bigdata, int icol);   // local size of backward rhok block in y-direction at col icol
int bigfft_np2loc_col(BIGFFT_DATA *bigdata, int icol);   // local size of backward rhok block in z-direction at col icol
int bigfft_origin_np0_bwd(BIGFFT_DATA *bigdata);   // absolute value of first local x-grid point
int bigfft_origin_np1_bwd(BIGFFT_DATA *bigdata);   // absolute value of first local y-grid point
int bigfft_origin_np1_fwd(BIGFFT_DATA *bigdata);   // absolute value of first local y-grid point
int bigfft_origin_np2_fwd(BIGFFT_DATA *bigdata);   // absolute value of first local z-grid point
int bigfft_nprow(BIGFFT_DATA *bigdata);
int bigfft_npcol(BIGFFT_DATA *bigdata);
int bigfft_mype(BIGFFT_DATA *bigdata);
int bigfft_myrow(BIGFFT_DATA *bigdata);
int bigfft_mycol(BIGFFT_DATA *bigdata);
MPI_Comm bigfft_comm(BIGFFT_DATA *bigdata);
int bigfft_gridpe_fwd(BIGFFT_DATA *bigdata, int x, int y, int z);

#endif
